// B-tree map of numbers to numbers -*- c++ -*-

#ifndef BTREE_H_
# define BTREE_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

/** @file BTree.h
 * B-tree map of numbers to numbers
 */

/** Size of the b-tree nodes (must be at least 8 and divisible by 4) */
# define BTREE_SIZE 512

# include <cstdio>
# include <cstring>

/** B-tree map of numbers to numbers */
class BTree
{
public:
  /** Type of the numbers to be stored */
  typedef unsigned item_t;
  /** Constructor
   * @param file	Disk file containing the B-tree
   */
  explicit BTree (FILE* file);
private:
  /** Copy constructor */
  BTree (const class BTree& old);
  /** Assignment operator */
  class BTree& operator= (const class BTree& old);
public:
  /** Destructor */
  ~BTree ();
  /** Search a tree
   * @param key		the key to be sought
   * @return		the values (item 0: amount of the values)
   */
  item_t* search (item_t key) const {
    return search (myRoot, key);
  }
  /** Insert an item to the tree
   * @param key		the key
   * @param value	the value
   */
  void insert (item_t key, item_t value);
private:
  /** Search the tree
   * @param page	node to start the search from
   * @param key		the key to be sought
   * @return		the values (item 0: amount of the values)
   */
  item_t* search (const item_t* page, item_t key) const;
  /** Search the tree
   * @param page	node to start the search from
   * @param key		the key to be sought
   * @return		the values (item 0: amount of the values)
   */
  item_t* search (item_t* page, item_t key) const;
  /** Split a child node
   * @param parent	the parent node
   * @param child	the child node
   * @param index	index of the child node in the parent node
   * @param childpage	page number of the parent node
   * @param childpage	page number of the child node
   */
  void split (item_t* parent, item_t* child,
	      unsigned index,
	      unsigned parentpage, unsigned childpage);
  /** Insert to a non-full node
   * @param pagenbr	page number of the node to insert to
   * @param key		the key
   * @param value	the value
   */
  void insert (unsigned pagenbr, item_t key, item_t value);

  /** The disk file containing the B-tree */
  FILE* myFile;
  /** Copy of the root node of the B-tree */
  item_t myRoot[BTREE_SIZE];
};

#endif // BTREE_H_
